unit OTFEFreeOTFE_frmWizardCreateVolumeAdvanced;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, SDUFilenameLabel, ComCtrls, Spin64;

type
  TfrmWizardCreateVolumeAdvanced = class(TForm)
    pcAdvancedOpts: TPageControl;
    tsSalt: TTabSheet;
    seSaltLength: TSpinEdit64;
    Label15: TLabel;
    Label18: TLabel;
    tsDriveLetter: TTabSheet;
    Label1: TLabel;
    cbDriveLetter: TComboBox;
    tsCDBLocation: TTabSheet;
    rbCDBInVolFile: TRadioButton;
    rbCDBInKeyfile: TRadioButton;
    gbKeyfile: TGroupBox;
    lblKeyFilename: TSDUFilenameLabel;
    pbBrowseKeyfile: TButton;
    pbOK: TButton;
    pbCancel: TButton;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    SaveDialog: TSaveDialog;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    tsKeyIterations: TTabSheet;
    Label16: TLabel;
    Label17: TLabel;
    Label20: TLabel;
    Label22: TLabel;
    seKeyIterations: TSpinEdit64;
    Label12: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure pbOKClick(Sender: TObject);
    procedure pbBrowseKeyfileClick(Sender: TObject);
  private
    function  GetKeyIterations(): integer;
    procedure SetKeyIterations(keyIterations: integer);
    function  GetSaltLength(): integer;
    procedure SetSaltLength(saltLength: integer);
    function  GetDriveLetter(): char;
    procedure SetDriveLetter(driveLetter: char);
    function  GetCDBFilename(): string;
    procedure SetCDBFilename(CDBFilename: string);
  protected
    procedure EnableDisableControls();

  public
    property KeyIterations: integer read GetKeyIterations write SetKeyIterations;
    property SaltLength: integer read GetSaltLength write SetSaltLength;
    property DriveLetter: char read GetDriveLetter write SetDriveLetter;
    property CDBFilename: string read GetCDBFilename write SetCDBFilename;
  end;


implementation

{$R *.dfm}

uses
  SDUGeneral,
  SDUDialogs,
  OTFEFreeOTFE_U;

procedure TfrmWizardCreateVolumeAdvanced.pbBrowseKeyfileClick(Sender: TObject);
begin
  SaveDialog.Filter     := FILE_FILTER_FLT_KEYFILES;
  SaveDialog.DefaultExt := FILE_FILTER_DFLT_KEYFILES;

  SDUOpenSaveDialogSetup(SaveDialog, CDBFilename);
  if SaveDialog.execute then
    begin
    CDBFilename := SaveDialog.Filename;
    end;

end;

procedure TfrmWizardCreateVolumeAdvanced.pbOKClick(Sender: TObject);
var
  allOK: boolean;
begin
  allOK:= TRUE;

  if allOK then
    begin
    // Ensure number of key iterations is > 0
    if (KeyIterations <= 0) then
      begin
      SDUMessageDlg('The number of key iterations must be 1 or more', mtError);
      pcAdvancedOpts.ActivePage := tsKeyIterations;
      ActiveControl := seKeyIterations;
      allOK := FALSE
      end;
    end;

  if allOK then
    begin
    // Ensure salt length is a multiple of 8 bits
    if ((SaltLength mod 8) <> 0) then
      begin
      SDUMessageDlg('The salt length must be a multiple of 8', mtError);
      pcAdvancedOpts.ActivePage := tsSalt;
      ActiveControl := seSaltLength;
      allOK := FALSE
      end;
    end;

  if allOK then
    begin
    // Either the user's selected to include the CDB in their volume file, or
    // they've specified a keyfilename
    allOK := rbCDBInVolFile.checked OR (rbCDBInKeyfile.checked AND (CDBFilename <> ''));
    if not(allOK) then
      begin
      SDUMessageDlg('If you wish to store the volume''s CDB in a keyfile, you must specify the file to use', mtError);
      pcAdvancedOpts.ActivePage := tsCDBLocation;
      allOK := FALSE
      end;
    end;


  if allOK then
    begin
    ModalResult := mrOK;
    end;

end;


function TfrmWizardCreateVolumeAdvanced.GetKeyIterations(): integer;
begin
  Result := seKeyIterations.Value;
end;

procedure TfrmWizardCreateVolumeAdvanced.SetKeyIterations(keyIterations: integer);
begin
  seKeyIterations.Value := keyIterations;
end;

function TfrmWizardCreateVolumeAdvanced.GetSaltLength(): integer;
begin
  Result := seSaltLength.Value;
end;

procedure TfrmWizardCreateVolumeAdvanced.SetSaltLength(saltLength: integer);
begin
  seSaltLength.Value := saltLength;
end;

function TfrmWizardCreateVolumeAdvanced.GetDriveLetter(): char;
var
  driveLetter: char;
begin
  driveLetter := #0;
  if (cbDriveLetter.ItemIndex > 0) then
    begin
    driveLetter := cbDriveLetter.Items[cbDriveLetter.ItemIndex][1];
    end;

  Result := driveLetter;
end;

procedure TfrmWizardCreateVolumeAdvanced.SetDriveLetter(driveLetter: char);
begin
//lplp
end;

procedure TfrmWizardCreateVolumeAdvanced.FormCreate(Sender: TObject);
var
  dl: char;
begin
  // tsKeyIterations
  seKeyIterations.MinValue := 1;
  seKeyIterations.MaxValue := 999999; // Need *some* upper value, otherwise setting MinValue won't work properly
  seKeyIterations.Increment := DEFAULT_KEY_ITERATIONS_INCREMENT;

  // tsSalt
  seSaltLength.Increment := 8;

  // tsRequestedDriveLetter
  cbDriveLetter.Items.Clear();
  cbDriveLetter.Items.Add('Use default');
  for dl:='C' to 'Z' do
    begin
    cbDriveLetter.Items.Add(dl+':');
    end;
  // Autoselect the "default" (first) entry
  cbDriveLetter.ItemIndex := 0;

  // tsCDBLocation
  rbCDBInKeyfile.checked := FALSE;
  rbCDBInVolFile.checked := TRUE;
  lblKeyFilename.Caption := '';


  // Set to first page
  pcAdvancedOpts.ActivePage := tsKeyIterations;
end;

function TfrmWizardCreateVolumeAdvanced.GetCDBFilename(): string;
var
  retval: string;
begin
  retval := '';
  if rbCDBInKeyfile.checked then
    begin
    retval := trim(lblKeyFilename.caption);
    end;

  Result := retval;
end;

procedure TfrmWizardCreateVolumeAdvanced.SetCDBFilename(CDBFilename: string);
begin
  rbCDBInVolFile.checked := TRUE;
  lblKeyFilename.caption := CDBFilename;
  if (CDBFilename <> '') then
    begin
    rbCDBInKeyfile.checked := TRUE;
    end;

end;


procedure TfrmWizardCreateVolumeAdvanced.EnableDisableControls();
begin
  SDUEnableControl(gbKeyfile, rbCDBInKeyfile.checked);
end;

END.


