unit SDUProgressDlg;
// Description: Progress bar dialog
// By Sarah Dean
// Email: sdean12@sdean12.org
// WWW:   http://www.SDean12.org/
//
// -----------------------------------------------------------------------------
//


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, ExtCtrls;

type
  TSDUProgressDialog = class(TForm)
    pnlProgressBar: TPanel;
    pgbOverall: TProgressBar;
    pbCancel: TButton;
    pnlStatusText: TPanel;
    lblStatus: TLabel;
    lblEstTimeRemainText: TLabel;
    lblEstTimeRemaining: TLabel;
    procedure pbCancelClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    fShowStatusText: boolean;
    fConfirmCancel: boolean;
    i64MinValue: int64;
    i64MaxValue: int64;
    i64PositionValue: int64;
    fShowTimeRemaining: boolean;
    fStartTime: TDateTime;

    procedure SetTitle(title: string);
    function  GetTitle(): string;

    procedure SetShowTimeRemaining(value: boolean);


    procedure SetShowStatusText(showStatusText: boolean);
    procedure SetStatusText(statusText: string);
    function  GetStatusText(): string;

    procedure UpdateI64ProgressBar();

    procedure iSetMax(max: integer);
    procedure iSetMin(min: integer);
    procedure iSetPosition(position: integer);
    procedure iSetInversePosition(position: integer);

    procedure i64SetMax(max: int64);
    procedure i64SetMin(min: int64);
    procedure i64SetPosition(position: int64);
    procedure i64SetInversePosition(position: int64);
  public
    Cancel: boolean;

    property  Title: string read GetTitle write SetTitle;
    property  ShowTimeRemaining: boolean read fShowTimeRemaining write SetShowTimeRemaining;

    property  Max: integer write iSetMax;
    property  Min: integer write iSetMin;
    property  Position: integer write iSetPosition;
    property  InversePosition: integer write iSetInversePosition;
    procedure IncPosition();

    property  i64Max: int64 read i64MaxValue write i64SetMax;
    property  i64Min: int64 read i64MinValue write i64SetMin;
    property  i64Position: int64 read i64PositionValue write i64SetPosition;
    property  i64InversePosition: int64 write i64SetInversePosition;
    procedure i64IncPosition();

    property  ConfirmCancel: boolean write fConfirmCancel default TRUE;

    property  ShowStatusText: boolean read fShowStatusText write SetShowStatusText;
    property  StatusText: string read GetStatusText write SetStatusText;
  end;


procedure Register;

implementation

{$R *.DFM}

uses
  SDUGeneral,
  math,
  DateUtils;

const
  TIME_REMAINING_UNKNOWN = '<unknown>';

procedure Register;
begin
  RegisterComponents('SDeanUtils', [TSDUProgressDialog]);
end;

procedure TSDUProgressDialog.SetTitle(title: string);
begin
  self.Caption := title;
end;

function  TSDUProgressDialog.GetTitle(): string;
begin
  Result := self.Caption;
end;

procedure TSDUProgressDialog.UpdateI64ProgressBar();
var
  barLength: int64;
  barPos: int64;
  percentPos: integer;
begin
  iSetMin(0);
  iSetMax(100);

  barLength := i64MaxValue - i64MinValue;
  barPos := i64PositionValue - i64MinValue;

  if (i64MaxValue<>0) then
    begin
    percentPos := ((barPos*100) div barLength);
    iSetPosition(percentPos);
    end;

end;



procedure TSDUProgressDialog.iSetMax(max: integer);
begin
  pgbOverall.Max := max;
end;

procedure TSDUProgressDialog.iSetMin(min: integer);
begin
  pgbOverall.Min := min;
end;

procedure TSDUProgressDialog.iSetPosition(position: integer);
var
  currTime: TDateTime;
  timeDiff: TDateTime;
  newLabel: string;
  AYear, AMonth, ADay, AHour, AMinute, ASecond, AMilliSecond: Word;
  timeRemaining: TDateTime;
begin
  if position>pgbOverall.Max then
    begin
    pgbOverall.position := pgbOverall.Max
    end
  else if position<pgbOverall.Min then
    begin
    pgbOverall.position := pgbOverall.Min
    end
  else
    begin
    pgbOverall.position := position;
    end;

  if fShowTimeRemaining then
    begin
    newLabel := TIME_REMAINING_UNKNOWN;

    currTime := Now;
    if (currTime <> fStartTime) then
      begin
      timeDiff := currTime - fStartTime;

      if ((pgbOverall.Max - pgbOverall.Min) <> 0) then
        begin
        timeRemaining := (timeDiff * ((pgbOverall.Max - pgbOverall.Position) / (pgbOverall.Max - pgbOverall.Min)));
        DecodeDateTime(
                       timeRemaining,
                       AYear,
                       AMonth,
                       ADay,
                       AHour,
                       AMinute,
                       ASecond,
                       AMilliSecond
                      );
        ADay := trunc(timeRemaining);        
        if (ADay > 0) then
          begin
          newLabel := Format(
                             '%d days, %d hours, %d minutes',
                             [ADay, AHour, AMinute]
                            );
          end
        else if (AHour > 0) then
          begin
          newLabel := Format(
                             '%d hours, %d minutes',
                             [AHour, AMinute]
                            );
          end
        else if (AMinute > 0) then
          begin
          newLabel := Format(
                             '%d minutes, %d seconds',
                             [AMinute, ASecond]
                            );
          end
        else
          begin
          newLabel := Format('%d seconds', [ASecond]);
          end;

        end;

      lblEstTimeRemaining.Caption := newLabel;
      end;
    end;

  self.refresh;
  Application.ProcessMessages();
end;

procedure TSDUProgressDialog.iSetInversePosition(position: integer);
begin
  iSetPosition(pgbOverall.Max-position);

end;

procedure TSDUProgressDialog.IncPosition();
begin
  iSetPosition(pgbOverall.Position+1);

end;


procedure TSDUProgressDialog.i64SetMax(max: int64);
begin
  i64MaxValue := max;
  UpdateI64ProgressBar();

end;

procedure TSDUProgressDialog.i64SetMin(min: int64);
begin
  i64MinValue := min;
  UpdateI64ProgressBar();

end;

procedure TSDUProgressDialog.i64SetPosition(position: int64);
begin
  i64PositionValue := position;
  UpdateI64ProgressBar();

end;

procedure TSDUProgressDialog.i64SetInversePosition(position: int64);
begin
  i64PositionValue := i64MaxValue-position;
  UpdateI64ProgressBar();

end;

procedure TSDUProgressDialog.i64IncPosition();
begin
  i64PositionValue := i64PositionValue + 1;
  UpdateI64ProgressBar();

end;


procedure TSDUProgressDialog.pbCancelClick(Sender: TObject);
begin
  Cancel := TRUE;
  pbCancel.Enabled := FALSE;
  if fConfirmCancel then
    begin
    showmessage('Operation cancelled by user');
    end;

end;

procedure TSDUProgressDialog.FormCreate(Sender: TObject);
begin
  pbCancel.Enabled := TRUE;
  Cancel := FALSE;

  pnlStatusText.Caption := '';
  pnlStatusText.BevelInner := bvNone;
  pnlStatusText.BevelOuter := bvNone;
  pnlProgressBar.Caption := '';
  pnlProgressBar.BevelInner := bvNone;
  pnlProgressBar.BevelOuter := bvNone;

  lblStatus.Caption := '';

  fShowStatusText := TRUE;  // The form is designed with it shown
  ShowStatusText := FALSE;  // Default to just the progress bar
  StatusText := '';

  lblEstTimeRemaining.Caption := TIME_REMAINING_UNKNOWN;
  lblEstTimeRemaining.Visible := FALSE;
  lblEstTimeRemainText.Visible := FALSE;

end;

procedure TSDUProgressDialog.SetShowStatusText(showStatusText: boolean);
begin
  if (fShowStatusText <> showStatusText) then
    begin
    if showStatusText then
      begin
      self.Height := self.Height + pnlStatusText.Height;
      pnlStatusText.Visible := TRUE;
      end
    else
      begin
      self.Height := self.Height - pnlStatusText.Height;
      pnlStatusText.Visible := FALSE;
      end;
    end;

  fShowStatusText := showStatusText;
end;

procedure TSDUProgressDialog.SetStatusText(statusText: string);
begin
  lblStatus.caption := statusText;

end;

function TSDUProgressDialog.GetStatusText(): string;
begin
  Result := lblStatus.caption;

end;

procedure TSDUProgressDialog.FormShow(Sender: TObject);
begin
  fStartTime := Now;
end;

procedure TSDUProgressDialog.SetShowTimeRemaining(value: boolean);
begin
  fShowTimeRemaining := value;

  lblEstTimeRemaining.Visible := fShowTimeRemaining;
  lblEstTimeRemainText.Visible := fShowTimeRemaining;
end;

END.

